package enseirb.myinpulse.controller;

import enseirb.myinpulse.model.Administrator;
import enseirb.myinpulse.model.Entrepreneur;
import enseirb.myinpulse.service.AdminApiService;
import enseirb.myinpulse.service.EntrepreneurApiService;
import enseirb.myinpulse.service.UtilsService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.web.bind.annotation.*;

@SpringBootApplication
@RestController
public class UnauthApi {

    private final EntrepreneurApiService entrepreneurApiService;
    private final UtilsService utilsService;

    @Autowired
    UnauthApi(EntrepreneurApiService entrepreneurApiService, UtilsService utilsService) {
        this.entrepreneurApiService = entrepreneurApiService;
        this.utilsService = utilsService;
    }

    @PostMapping("/unauth/finalize")
    public void createAccount(@AuthenticationPrincipal Jwt principal) {
        boolean sneeStatus;
        if (principal.getClaimAsString("sneeStatus") != null) {
            sneeStatus = principal.getClaimAsString("sneeStatus").equals("true");
        } else {
            sneeStatus = false;
        }
        String userSurname = principal.getClaimAsString("userSurname");
        String username = principal.getClaimAsString("preferred_username");
        String primaryMail = principal.getClaimAsString("email");
        String secondaryMail = principal.getClaimAsString("secondaryMail");
        String phoneNumber = principal.getClaimAsString("phoneNumber");
        String school = principal.getClaimAsString("school");
        String course = principal.getClaimAsString("course");
        Entrepreneur e =
                new Entrepreneur(
                        userSurname,
                        username,
                        primaryMail,
                        secondaryMail,
                        phoneNumber,
                        school,
                        course,
                        sneeStatus,
                        true);

        entrepreneurApiService.createAccount(e);
    }

    @GetMapping("/unauth/check-if-not-pending")
    public Boolean checkAccountStatus(@AuthenticationPrincipal Jwt principal) {
        // Throws 404 if user not found
        return utilsService.checkEntrepreneurNotPending(principal.getClaimAsString("email"));
    }
}
